﻿#region File Header

/* File Name    : FacebookManager.cs
 * Description  : This class contains methods to handle Facebook requests.
 * Log
 * Date             Author              Description
 * 15-Oct-2011      Robin Jome          Created the class
 * 19-Oct-2011      Robin Jome          New Method GetUrl() added
 */

#endregion

#region Namespaces

using System;
using System.Collections.Generic;
using FacebookAPI.FacebookProperties;

#endregion

namespace FacebookAPI.Base
{
    #region Class

    /// <summary>
    /// This class contains methods to handle Facebook requests.
    /// </summary>
    internal class FacebookManager
    {
        #region Common methods

        /// <summary>
        /// Prepares a Facebook Uri object based on the inputs provided
        /// </summary>
        /// <param name="objectId">The Object Id/Name</param>
        /// <param name="connectionType">The type of Facebook connection</param>
        /// <param name="fields">The fields to be retrieved/submitted</param>
        /// <param name="searchOptions">The search/filter options to be used</param>
        /// <param name="accessToken">The Facebook access token</param>
        /// <param name="otherParameters">Additional parameters to be used.</param>
        /// <param name="isIntrospection">True if the request need to be an introspection request.</param>
        /// <returns>Returns a FacebookUrl object</returns>
        public FacebookUrl GetUrl(string objectId, Nullable<FacebookConnections> connectionType, FacebookQueryField fields, SearchOptions searchOptions, string accessToken, List<string> otherParameters, bool isIntrospection)
        {
            return new FacebookUrl()
            {
                ObjectId = objectId,
                ConnectionType = connectionType,
                AccessToken = accessToken,
                SearchOptions = searchOptions,
                IsIntrospection = isIntrospection,
                Fields = fields,
                OtherParameters = otherParameters
            };
        }

        /// <summary>
        /// Prepares a Facebook Uri object based on the inputs provided
        /// </summary>
        /// <param name="objectId">The Object Id/Name</param>
        /// <param name="connectionType">The type of Facebook connection</param>
        /// <param name="fields">The fields to be retrieved/submitted</param>
        /// <param name="searchOptions">The search/filter options to be used</param>
        /// <param name="accessToken">The Facebook access token</param>
        /// <param name="isIntrospection">True if the request need to be an introspection request.</param>
        /// <returns>Returns a FacebookUrl object</returns>
        public FacebookUrl GetUrl(string objectId, Nullable<FacebookConnections> connectionType, FacebookQueryField fields, SearchOptions searchOptions, string accessToken, bool isIntrospection)
        {
            return new FacebookUrl()
            {
                ObjectId = objectId,
                ConnectionType = connectionType,
                AccessToken = accessToken,
                SearchOptions = searchOptions,
                IsIntrospection = isIntrospection,
                Fields = fields,
            };
        }

        /// <summary>
        /// Prepares a Facebook Uri object based on the inputs provided
        /// </summary>
        /// <param name="objectId">The Object Id/Name</param>
        /// <param name="connectionType">The type of Facebook connection</param>
        /// <param name="accessToken">The Facebook access token</param>
        /// <returns>Returns a FacebookUrl object</returns>
        public FacebookUrl GetUrl(string objectId, Nullable<FacebookConnections> connectionType, string accessToken)
        {
            return GetUrl(objectId, connectionType, null, null, accessToken, false);
        }

        #endregion

        #region Data Retrieval methods

        /// <summary>
        /// Retrieves a object with default/specified fields providing the object name/id
        /// </summary>
        /// <typeparam name="FBObject">The Facebook object to be retrieved.</typeparam>
        /// <param name="url">The Facebook Uri to be executed.</param>
        /// <returns>Returns a Facebook object</returns>
        internal FBObject GetObject<FBObject>(FacebookUrl url)
        {
            FBObject fbObject = default(FBObject);
            FacebookRequestHandler handler = new FacebookRequestHandler();
            fbObject = handler.ExecuteGetRequest<FBObject>(url);
            return fbObject;
        }

        /// <summary>
        /// Gets the response from Facebook server without de-serializing the output.
        /// </summary>
        /// <param name="url">The url to be executed</param>
        /// <returns>Returns the output of Facebook url execution.</returns>
        internal string GetResponse(FacebookUrl url)
        {
            FacebookRequestHandler handler = new FacebookRequestHandler();
            return handler.ExecuteGetRequest(url);
        }

        /// <summary>
        /// Retrieves a list of Facebook objects with default/specified fields.
        /// </summary>
        /// <typeparam name="FBObject">The Facebook object to be retrieved.</typeparam>
        /// <param name="url">The Facebook Uri to be executed.</param>
        /// <returns>Returns a Facebook connection object containing a list of Facebook objects.</returns>
        internal FacebookConnection<ConnectionType> GetConnectionObject<ConnectionType>(FacebookUrl url) where ConnectionType : class
        {
            return GetObject<FacebookConnection<ConnectionType>>(url);
        }

        #endregion

        #region Data submission methods

        /// <summary>
        /// Saves a Facebook object
        /// </summary>
        /// <typeparam name="FBReturnObject">The type of object returned after saving the Facebook object</typeparam>
        /// <param name="postData">The data to be submitted in the post request as a string.</param>
        /// <param name="url">The Facebook Uri to be submitted.</param>
        /// <returns>Returns the result of the save action as FBReturnObject</returns>
        internal FBReturnObject SaveObject<FBReturnObject>(FacebookUrl url, string postData)
        {
            FacebookRequestHandler handler = new FacebookRequestHandler();
            FBReturnObject responseData = handler.ExecutePostRequest<FBReturnObject>(url, postData);
            return responseData;
        }

        /// <summary>
        /// Saves a Facebook object
        /// </summary>
        /// <typeparam name="FBReturnObject">The type of object returned after saving the Facebook object</typeparam>
        /// <param name="url">The Facebook Uri to be submitted.</param>
        /// <returns>Returns the result of the save action as FBReturnObject</returns>
        internal FBReturnObject SaveObject<FBReturnObject>(FacebookUrl url)
        {
            return SaveObject<FBReturnObject>(url, null);
        }

        /// <summary>
        /// Saves list of files passed.
        /// </summary>
        /// <param name="url">The Facebook Uri to be submitted.</param>
        /// <param name="fileNames">The list of file names to be saved.</param>
        internal void SaveFile(FacebookUrl url, List<string> fileNames)
        {
            FacebookRequestHandler handler = new FacebookRequestHandler();
            handler.SaveFiles(url, fileNames);
        }

        #endregion

        #region Delete Methods

        /// <summary>
        /// Deletes a Facebook object
        /// </summary>
        /// <param name="url">The Facebook Uri to be submitted.</param>
        /// <returns>Returns the result of the delete operation.</returns>
        internal FBReturnObject DeleteObject<FBReturnObject>(FacebookUrl url)
        {
            FacebookRequestHandler handler = new FacebookRequestHandler();
            if (!url.OtherParameters.Contains("method=DELETE"))
                url.OtherParameters.Add("method=DELETE");
            return handler.ExecuteGetRequest<FBReturnObject>(url);
        }

        #endregion
    }

    #endregion
}
