﻿#region File Header

/* File Name    : FacebookRequestHandler.cs
 * Description  : This class contains methods to execute a Facebook web request.
 * Log
 * Date             Author              Description
 * 15-Oct-2011      Robin Jome          Created the class
 * 16-Oct-2011      Robin Jome          Removed out parameters from methods and added FacebookException
 * 17-Oct-2011      Robin Jome          Updated ExecutePostRequest().
 */

#endregion

#region Namespaces

using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Text;
using System.Web.Script.Serialization;
using FacebookAPI.FacebookProperties;

#endregion

namespace FacebookAPI.Base
{
    #region Class

    /// <summary>
    /// This class contains methods to get and post data to and from Facebook
    /// </summary>
    internal class FacebookRequestHandler
    {
        #region Members

        /// <summary>
        /// The Json Serializer objects
        /// </summary>
        JavaScriptSerializer serializer;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes the FacebookRequestHandler class
        /// </summary>
        public FacebookRequestHandler()
        {
            serializer = new JavaScriptSerializer();
        }

        #endregion

        #region Methods

        #region ExecutePostRequest

        /// <summary>
        /// Function used to send a POST request which returns an output value.
        /// </summary>
        /// <typeparam name="FBReturnObject">Represents the type of object to be returned</typeparam>
        /// <param name="url">Represents the Uri to which the post data shall be submitted.</param>
        /// <param name="postData">Represents the Post Data in string format to be submitted.</param>
        /// <param name="fileList">The optional list of files to be saved</param>
        /// <returns>Returns the result of Post submission operation.</returns>
        internal FBReturnObject ExecutePostRequest<FBReturnObject>(FacebookUrl url, string postData)
        {
            FBReturnObject responseData = default(FBReturnObject);
            WebRequest webRequest = GetWebRequest(url);
            webRequest.Method = "POST";
            webRequest.ContentType = "application/x-www-form-UrlEncoded";

            byte[] buffer = null;
            if (!string.IsNullOrEmpty(postData))
            {
                buffer = Encoding.UTF8.GetBytes(postData);
                webRequest.ContentLength = buffer.Length;
            }
            try
            {
                using (Stream stream = webRequest.GetRequestStream())
                {
                    if (buffer != null)
                    {
                        stream.Write(buffer, 0, buffer.Length);
                    }
                    using (WebResponse webResponse = webRequest.GetResponse())
                    {
                        if (webResponse != null)
                        {
                            using (StreamReader streamReader = new StreamReader(webResponse.GetResponseStream(), Encoding.UTF8))
                            {
                                string data = streamReader.ReadToEnd();
                                if (!string.IsNullOrEmpty(data))
                                {
                                    responseData = serializer.Deserialize<FBReturnObject>(data);
                                }
                            }
                        }
                    }
                }
            }
            catch (WebException ex)
            {
                throw new FacebookException(ex.Message, ex.InnerException, ex.Status, ex.Response, GetFacebookError(ex));
            }
            return responseData;
        }

        #endregion

        #region SaveFile

        internal void SaveFiles(FacebookUrl url, List<string> fileNames)
        {
            try
            {
                WebClient client = new WebClient();
                foreach (string file in fileNames)
                {
                    client.UploadFile(url.GetUrl(), file);
                }
            }
            catch (WebException ex)
            {
                throw new FacebookException(ex.Message, ex.InnerException, ex.Status, ex.Response, GetFacebookError(ex));
            }
        }

        #endregion

        #region ExecuteGetRequest

        /// <summary>
        /// This method gets data from a specified Uri
        /// </summary>
        /// <typeparam name="FBObject">Represents the Type of data to be processed by the request.</typeparam>
        /// <param name="url">Represents the Uri from which data shall be fetched.</param>
        /// <returns>Represents the data output produced on successful completion of the request.</returns>
        internal FBObject ExecuteGetRequest<FBObject>(FacebookUrl url)
        {
            FBObject responseData = default(FBObject);
            JavaScriptSerializer serializer = new JavaScriptSerializer();
            WebRequest webRequest = GetWebRequest(url);
            try
            {
                using (WebResponse webResponse = webRequest.GetResponse())
                {
                    if (webResponse != null)
                    {
                        using (StreamReader streamReader = new StreamReader(webResponse.GetResponseStream(), Encoding.UTF8))
                        {
                            string data = streamReader.ReadToEnd();
                            if (!string.IsNullOrEmpty(data))
                            {
                                responseData = serializer.Deserialize<FBObject>(data);
                            }
                        }
                    }
                }
            }
            catch (WebException ex)
            {
                throw new FacebookException(ex.Message, ex.InnerException, ex.Status, ex.Response, GetFacebookError(ex));
            }
            catch (Exception ex)
            {
                throw new FacebookException("Some error occurred while reading data from Facebook", ex);
            }
            return responseData;
        }

        /// <summary>
        /// This method gets data as string from a specified Uri
        /// </summary>
        /// <param name="url">Represents the Uri from which data shall be fetched.</param>
        /// <returns>Represents the data output produced as string on successful completion of the request.</returns>
        internal string ExecuteGetRequest(FacebookUrl url)
        {
            string data = string.Empty;
            WebRequest webRequest = GetWebRequest(url);
            try
            {
                using (WebResponse webResponse = webRequest.GetResponse())
                {
                    if (webResponse != null)
                    {
                        using (StreamReader streamReader = new StreamReader(webResponse.GetResponseStream(), Encoding.UTF8))
                        {
                            data = streamReader.ReadToEnd();
                        }
                    }
                }
            }
            catch (WebException ex)
            {
                throw new FacebookException(ex.Message, ex.InnerException, ex.Status, ex.Response, GetFacebookError(ex));
            }
            catch (Exception ex)
            {
                throw new FacebookException("Some error occurred while reading data from Facebook", ex);
            }
            return data;
        }

        #endregion

        #region Common Methods

        /// <summary>
        /// This method returns a WebRequest object constructed using the given Uri and values.
        /// </summary>
        /// <param name="url">Represents the Uri to be used to construct the WebRequest.</param>
        /// <param name="values">Represents an array of values to be used to construct the Uri</param>
        /// <returns>Returns a WebRequest object.</returns>
        private WebRequest GetWebRequest(FacebookUrl url)
        {
            WebRequest webRequest = WebRequest.Create(url.GetUrl());
            webRequest.Proxy = HttpWebRequest.DefaultWebProxy;
            webRequest.Proxy.Credentials = CredentialCache.DefaultCredentials;
            return webRequest;
        }

        /// <summary>
        /// This method gets a Facebook error object from the web response
        /// </summary>
        /// <param name="ex">The WebException object</param>
        /// <returns>Returns a Facebook error object</returns>
        private FacebookError GetFacebookError(WebException ex)
        {
            FacebookError error = null;
            try
            {
                using (WebResponse response = ex.Response)
                {
                    using (Stream data = response.GetResponseStream())
                    {
                        string text = new StreamReader(data).ReadToEnd();
                        error = serializer.Deserialize<FacebookError>(text);
                    }
                }
            }
            catch { }
            return error;
        }

        #endregion

        #endregion
    }

    #endregion
}
