﻿#region File Header

/* File Name    : FacebookEvent.cs
 * Description  : Specifies information about an event, including the location, event name
 * Log
 * Date             Author              Description
 * 15-Oct-2011      Robin Jome          Created the class
 * 17-Oct-2011      Robin Jome          Updated comments.
 *                                      Removed ToString() overload and added GetPostData() method
 */

#endregion

#region Namespaces

using System;
using System.Text;
using FacebookAPI.FacebookProperties;

#endregion

namespace FacebookAPI.Objects
{
    #region Class

    /// <summary>
    /// Specifies information about an event, including the location, event name
    /// </summary>
    /// <permission>Generic access_token, user_events or friends_events</permission>
    /// <remarks>Specifies information about an event, including the location, event name, and which invitees plan to attend. The User, Page, and Application objects have an events connection.
    /// To read an Event, you need:    
    /// 1) A generic access_token for public events (those whose privacy is set to OPEN)
    /// 2) The user_events permission for non-public events
    /// 3) The friends_events permission for non-public events of the user's friends
    /// </remarks>
    public class FacebookEvent : FacebookBaseObject
    {
        #region Properties

        /// <summary>
        /// Gets the value of Owner
        /// </summary>
        /// <value>The profile that created the event</value>
        /// <permission>generic access_token, user_events or friends_events</permission>
        public FacebookBaseObject Owner { get; set; }

        /// <summary>
        /// Gets or sets the value of Description
        /// </summary>
        /// <value>The long-form description of the event</value>
        /// <permission>generic access_token, user_events or friends_events</permission>
        public string Description { get; set; }

        /// <summary>
        /// Gets or sets the value of Start_Time
        /// </summary>
        /// <value>The start time of the event, as you want it to be displayed on Facebook</value>
        /// <permission>generic access_token, user_events or friends_events</permission>
        public DateTime Start_Time { get; set; }

        /// <summary>
        /// Gets or sets the value of End_Time
        /// </summary>
        /// <value>The end time of the event, as you want it to be displayed on Facebook.</value>
        /// <permission>generic access_token, user_events or friends_events</permission>
        public DateTime End_Time { get; set; }

        /// <summary>
        /// Gets or sets the value of Location
        /// </summary>
        /// <value>The location for this event</value>
        /// <permission>generic access_token, user_events or friends_events</permission>
        public string Location { get; set; }

        /// <summary>
        /// Gets or sets the value of Privacy
        /// </summary>
        /// <value>The visibility of this event ['OPEN', 'CLOSED', or 'SECRET']</value>
        /// <permission>generic access_token, user_events or friends_events</permission>
        public FacebookPrivacyTypes Privacy { get; set; }

        /// <summary>
        /// Gets the value of Updated_Time
        /// </summary>
        /// <value>The last time the event was updated</value>
        /// <permission>generic access_token, user_events or friends_events</permission>
        public DateTime Updated_Time { get; set; }

        #endregion

        #region Methods

        /// <summary>
        /// Returns the data to be submitted in a POST request.
        /// </summary>
        /// <returns>Returns the data to be submitted in a POST request.</returns>
        internal string GetPostData()
        {
            StringBuilder postData = new StringBuilder();
            postData.Append("name=").Append(Name);
            postData.Append("&privacy_type=").Append(Enum.GetName(typeof(FacebookPrivacyTypes), Privacy));
            if (Start_Time != DateTime.MinValue)
                postData.Append("&start_time=").Append(Start_Time);
            if (End_Time != DateTime.MinValue)
                postData.Append("&end_date=").Append(End_Time);
            postData.Append("&description=").Append(Description);
            if (!string.IsNullOrEmpty(Location))
                postData.Append("&location=").Append(Location);
            return postData.ToString();
        }

        #endregion
    }

    #endregion
}
